# meendag - An unofficial SDK for Meendag.com help you to find name of the owner of the number
# Copyright (C) 2022 - Awiteb <awiteb@hotmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from requests import request, Response


url = "https://meendag.com/"
lookup_page = url + "lookup"

__all__ = ["countrys", "get_name"]


def send_request(url: str, method: str, **kwargs) -> Response:
    """Return response of `url` with `method` and pyload `kwargs`

    Args:
        url (str): Url you want send request to it
        method (str): Method of request `GET` `POST` Extra

    Returns:
        Response: The response
    """
    method = method.upper()
    response = request(
        method,
        url,
        params=kwargs if method == "GET" else None,
        data=None if method == "GET" else kwargs,
    )
    return response


def countrys() -> Response:
    """Returns countrys response

    Returns:
        Response: countrys response
    """
    return send_request(url, method="GET")


def get_name(number: str, country_code: str) -> Response:
    """Return response of lookup page

    Args:
        number (str): The number
        country_code (str): The country code

    Returns:
        Response: response of lookup page
    """
    return send_request(
        lookup_page, method="POST", num=number, countryCode=country_code
    )



# meendag - An unofficial SDK for Meendag.com help you to find name of the owner of the number
# Copyright (C) 2022 - Awiteb <awiteb@hotmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from typing import List, Optional
from .utils import *
from .types import Country, Owner
from .apihelper import countrys, get_name

__all__ = [
    "get_countrys",
    "get_owner",
    "get_country_by_code",
    "get_country_by_name",
    "countrys_filter",
]


def get_countrys() -> List[Country]:
    """Returns all countrys from meendag

    Returns:
        List[country]: countrys, {country_code: country_name}

    ## Example
    ```python
    >>> from meendag import get_countrys
    >>> countrys = get_countrys()
    >>> for country in countrys:
        print(
            f"Country name: {country.name} {country.code}"
        )
    Country name: المملكة العربية السعودية SA
    Country name: الأردن JO
    Country name: الامارات العربية المتحدة AE
    Country name: البحرين BH
    ...
    ```
    """
    return [
        Country(code=code, name=name)
        for code, name in parse_countrys(countrys()).items()
    ]


def get_owner(number: str, country: Country) -> Optional[Owner]:
    """Returns the name of the owner of the number, if any

    Args:
        number (str): The number
        country (Country): Country of number

    Returns:
        Optional[str]: name of the owner of the number, if any

    ## Example
    ```python
    >>> from meendag import get_country_by_code, get_owner
    >>> get_owner("0138823616", get_country_by_code("SA"))
    Owner(name='Intertek', number='0138823616', country=Country(code='SA', name='المملكة العربية السعودية'))
    ```
    """
    name = parse_name(get_name(number, country.code))
    if name:
        return Owner(name=name, number=number, country=country)


def countrys_filter(func) -> Optional[List[Country]]:
    """Filter countrys by `func`

    Args:
        func (Function): function to filter countrys

    Returns:
        Optional[Country]: Country by filter

    ## Example
    ```python
    >>> from meendag import countrys_filter
    >>> print(countrys_filter(lambda country: country.code == "SA"))
    [Country(code='SA', name='المملكة العربية السعودية')]
    ```
    """
    countrys = list(filter(func, get_countrys()))
    if countrys:
        return countrys


def get_country_by_code(country_code: str) -> Optional[Country]:
    """Returns country by country code

    Args:
        country_code (str): country code

    Returns:
        Optional[Country]: country

    ## Example
    ```python
    >>> from meendag import get_country_by_code
    >>> get_country_by_code("SA")
    Country(code='SA', name='المملكة العربية السعودية')
    ```
    """
    func = lambda country: country.code.lower() == country_code.lower()
    if country := countrys_filter(func):
        return country[0]


def get_country_by_name(country_name: str) -> Optional[Country]:
    """Returns country by country name

    Args:
        country_name (str): country name

    Returns:
        Optional[Country]: country

    ## Example
    ```python
    >>> from meendag import get_country_by_name
    >>> get_country_by_name("المملكة العربية السعودية")
    Country(code='SA', name='المملكة العربية السعودية')
    ```
    """
    func = lambda country: country.name == country_name
    if country := countrys_filter(func):
        return country[0]



# meendag - An unofficial SDK for Meendag.com help you to find name of the owner of the number
# Copyright (C) 2022 - Awiteb <awiteb@hotmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


from . import types, utils
from .main import *

version = "0.0.0"

__author__ = "Awiteb <awiteb@hotmail.com>"
__email__ = "awiteb@gmail.com"
__license__ = "AGPL"
__version__ = version



# meendag - An unofficial SDK for Meendag.com help you to find name of the owner of the number
# Copyright (C) 2022 - Awiteb <awiteb@hotmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from pydantic import BaseModel

__all__ = ["Country", "Owner"]


class Country(BaseModel):

    code: str
    """ Country code """

    name: str
    """ Country name """


class Owner(BaseModel):

    name: str
    """ Owner name """

    number: str
    """ Owner number """

    country: Country
    """ Country of number """



from setuptools import setup, find_packages
import re

KEYWORD = ["meendag", "Meendag", "number-owner"]

with open("meendag/__init__.py", "r", encoding="utf-8") as f:
    version = re.search(
        r'^version\s*=\s*"(.*)".*$', f.read(), flags=re.MULTILINE
    ).group(1)

with open("README.md", "r", encoding="utf-8") as readme_file:
    long_description = readme_file.read()

with open("requirements.txt", encoding="utf-8") as require_file:
    requires = [require.strip() for require in require_file]

setup(
    name="meendag",
    version=version,
    description="An unofficial SDK for Meendag.com help you to find name of the owner of the number",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Awiteb",
    author_email="Awiteb@hotmail.com",
    url="https://github.com/TheAwiteb/meendag",
    packages=find_packages(),
    license="AGPL",
    keywords=KEYWORD,
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
    install_requires=requires,
)



# meendag - An unofficial SDK for Meendag.com help you to find name of the owner of the number
# Copyright (C) 2022 - Awiteb <awiteb@hotmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from requests import Response
from bs4 import BeautifulSoup as bs4
from typing import Dict, Optional

__all__ = [
    "parse_countrys",
    "parse_name",
]


def parse_countrys(response: Response) -> Dict[str, str]:
    """Return countrys from response

    Args:
        response (Response): Response have the countrys

    Returns:
        Dict[str, str]: countrys, {country_code: country_name}
    """
    soup = bs4(response.content, "html.parser")
    countrys = {
        lang.attrs["value"]: lang.text
        for lang in soup.find(attrs={"name": "countryCode"}).find_all("option")
    }
    return countrys


def parse_name(response: Response) -> Optional[str]:
    """Return name from response

    Args:
        response (Response): Response have the name

    Returns:
        Optional[str]: name if any
    """
    soup = bs4(response.content, "html.parser")
    name = soup.find(class_="contactName").text.strip()

    if name != "ما لقيت الرقم معليش":
        return name
